#!/usr/bin/python
#
# Defaults script; update some default values
#
# Copyright (c) 2011 aizo ag, Switzerland
# Author: Johannes Winkelmann, johannes.winkelmann@aizo.com

import sys
import getopt
from dsmapi2 import *

CONNECTION = "tcp://localhost:8442"
TIMEOUT = 10

def listToDSID(l):
    return ':'.join(["%02X" % x for x in l])

def getDsmList(handle):
    dsms = DsmApiGetBusMembers(handle, 63)
    return filter(DsmApiIsdSM, dsms[1:])
        
def configurePing(handle):
    retval = 0
    deviceCount = 0
    print "> Configuring ping settings..."
    for dsm in getDsmList(handle):
        flags = dSMProperties_get_flags(handle, dsm)[1]
        flags = flags | 6
        if dSMProperties_set_flags(handle, dsm, flags):
            print ">> set flags failed"
            retval = 1

        result = dSMProperties_set_ping_intervals(handle, dsm, 20000, 60000)
        if result is not 0:
            print ">> Setting the ping intervals failed"
            retval = 1

        deviceCount = deviceCount + 1
        
    print "> done (%d dSMs)" % deviceCount
    return retval

def configureTemperatureLimits(handle, warnTemperature, errorTemperature):
    retval = 0
    deviceCount = 0
    print "> Configuring temperature settings..."
    for dsm in getDsmList(handle):
        result = dSMProperties_set_temperature_limits(handle, dsm,
                                                      warnTemperature, errorTemperature)
        if result is not 0:
            print ">> setting temperature limits failed"
            retval = 2
        deviceCount = deviceCount + 1
    print "> done (%d dSMs)" % deviceCount
    return retval

def configureChip(handle, configClass, offset, value):
    retval = 0
    count = 0
    print "> Configuring chip settings...  (Bank 0x%02x, Offset 0x%02x, Value 0x%02x)" % \
          (configClass, offset, value)
    for dsm in getDsmList(handle):
        deviceCount = ZoneDeviceCount_only_active(handle, dsm, 0)[1]
        print "\n>> Configuring %d devices on %s..." % (deviceCount, listToDSID(dsm))
        for j in range(deviceCount):
            resultArray = DeviceInfo_by_index_only_active(handle, dsm, 0, j)
            succ, deviceID = resultArray[:2]
            vendorID = resultArray[2]
            serialNumber = resultArray[15]

            dsidString = listToDSID(DsmApiExpandDeviceDSID(vendorID, serialNumber)[1])
            result = DeviceConfig_set(handle, dsm, deviceID, configClass, offset, value)
            if result is not 0:
                print "\n>>> setting device config failed on device %s" % dsidString
                retval = 4
            result, setValue = DeviceConfig_get_sync_8(handle, dsm, deviceID, configClass,
                                                       offset, TIMEOUT)[0:2]
            if result is not 0:
                print "\n>>> reading device config failed on device %s" % dsidString
                retval = 4
            elif setValue != value:
                print "\n>>> Failed to set param on device %s" % dsidString
                retval = 4
            sys.stdout.write('.')
            sys.stdout.flush()

            count = count + 1
    print "\n> done (%d devices)" % count
    return retval


#######

doChip = doPing = doTemp = False

opts, args = getopt.getopt(sys.argv[1:], 'cptn')
for o, a in opts:
    if o == '-c':
        doChip = True
    elif o == '-p':
        doPing = True
    elif o == '-t':
        doTemp = True
    elif o == '-n':
        nonVerbose = True

if len(args) > 0:
    CONNECTION = args[0]

if not doChip and not doPing and not doTemp:
    print "Usage: %s [options]" % sys.argv[0]
    print "    where options can be:"
    print "    -c configure chip settings"
    print "    -p configure ping settings"
    print "    -t configure temperature settings"
    sys.exit(0)



handle = DsmApiInitialize()
if DsmApiOpen(handle, CONNECTION, 0) != 0:
    print "Could not open DsmApi on %s" % CONNECTION
    sys.exit(1)

retval = 0
if doChip:
    retval |= configureChip(handle, 0x4, 0x0c, 0x76) # FKE2
    retval |= configureChip(handle, 0x4, 0x14, 0x14) # MINBITLEN
if doPing:
    retval |= configurePing(handle)
if doTemp:
    retval |= configureTemperatureLimits(handle, 90, 150)

DsmApiClose(handle)

sys.exit(retval)
